@@ -33,6 +33,7 @@
 #include <KToggleFullScreenAction>
 #include <KWindowEffects>
 #include <KWindowSystem>
+#include <KX11Extras>
 
 #include <QApplication>
 #include <QDBusConnection>
@@ -185,6 +186,8 @@ void MainWindow::initWaylandSurface()
     if (auto surface = KWayland::Client::Surface::fromWindow(windowHandle())) {
         m_plasmaShellSurface = m_plasmaShell->createSurface(surface, this);
         m_plasmaShellSurface->setPosition(pos());
+        m_plasmaShellSurface->setSkipTaskbar(true);
+        m_plasmaShellSurface->setSkipSwitcher(true);
     }
 }
 
@@ -248,11 +251,7 @@ void MainWindow::setupActions()
     action = actionCollection()->addAction(QStringLiteral("toggle-window-state"));
     action->setText(xi18nc("@action", "Open/Retract Yakuake"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("yakuake")));
-#ifndef Q_OS_WIN /* PORT */
     KGlobalAccel::self()->setGlobalShortcut(action, QList<QKeySequence>() << QKeySequence(Qt::Key_F12));
-#else
-    KGlobalAccel::self()->setGlobalShortcut(action, QList<QKeySequence>() << QKeySequence(Qt::Key_F11));
-#endif
     connect(action, SIGNAL(triggered()), this, SLOT(toggleWindowState()));
     connect(action, SIGNAL(changed()), this, SLOT(updateTrayTooltip()));
     connect(KGlobalAccel::self(), SIGNAL(globalShortcutChanged(QAction *, const QKeySequence &)), this, SLOT(updateTrayTooltip()));
@@ -325,13 +324,13 @@ void MainWindow::setupActions()
     action = actionCollection()->addAction(QStringLiteral("previous-session"));
     action->setText(xi18nc("@action", "Previous Session"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("go-previous")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::SHIFT + Qt::Key_Left));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::SHIFT | Qt::Key_Left));
     connect(action, SIGNAL(triggered()), m_tabBar, SLOT(selectPreviousTab()));
 
     action = actionCollection()->addAction(QStringLiteral("next-session"));
     action->setText(xi18nc("@action", "Next Session"));
     action->setIcon(QIcon::fromTheme(QStringLiteral("go-next")));
-    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::SHIFT + Qt::Key_Right));
+    actionCollection()->setDefaultShortcut(action, QKeySequence(Qt::SHIFT | Qt::Key_Right));
     connect(action, SIGNAL(triggered()), m_tabBar, SLOT(selectNextTab()));
 
     action = actionCollection()->addAction(QStringLiteral("move-session-left"));
@@ -738,7 +737,7 @@ void MainWindow::updateWindowWidthMenu()
 
     if (m_windowWidthMenu->isEmpty()) {
         for (int i = 10; i <= 100; i += 10) {
-            action = m_windowWidthMenu->addAction(QString::number(i) + QStringLiteral("%"));
+            action = m_windowWidthMenu->addAction(i18n("%1%", i));
             action->setCheckable(true);
             action->setData(i);
             action->setChecked(i == Settings::width());
@@ -760,7 +759,7 @@ void MainWindow::updateWindowHeightMenu()
 
     if (m_windowHeightMenu->isEmpty()) {
         for (int i = 10; i <= 100; i += 10) {
-            action = m_windowHeightMenu->addAction(QString::number(i) + QStringLiteral("%"));
+            action = m_windowHeightMenu->addAction(i18n("%1%", i));
             action->setCheckable(true);
             action->setData(i);
             action->setChecked(i == Settings::height());
@@ -842,8 +841,8 @@ void MainWindow::configureApp()
 
     connect(settingsDialog, &QDialog::finished, [=]() {
         m_toggleLock = true;
-        KWindowSystem::activateWindow(winId());
-        KWindowSystem::forceActiveWindow(winId());
+        KWindowSystem::activateWindow(windowHandle());
+        KX11Extras::forceActiveWindow(winId());
     });
 
     settingsDialog->show();
@@ -926,13 +925,20 @@ void MainWindow::applySkin()
 
 void MainWindow::applyWindowProperties()
 {
-    if (Settings::keepOpen() && !Settings::keepAbove()) {
-        KWindowSystem::clearState(winId(), NET::KeepAbove);
-        KWindowSystem::setState(winId(), NET::Sticky | NET::SkipTaskbar | NET::SkipPager);
-    } else
-        KWindowSystem::setState(winId(), NET::KeepAbove | NET::Sticky | NET::SkipTaskbar | NET::SkipPager);
+    if (m_isX11) {
+        if (Settings::keepOpen() && !Settings::keepAbove()) {
+            KWindowSystem::clearState(winId(), NET::KeepAbove);
+            KWindowSystem::setState(winId(), NET::SkipTaskbar | NET::SkipPager);
+        } else
+            KWindowSystem::setState(winId(), NET::KeepAbove | NET::SkipTaskbar | NET::SkipPager);
+    }
 
-    KWindowSystem::setOnAllDesktops(winId(), Settings::showOnAllDesktops());
+    if (m_isWayland && m_plasmaShellSurface) {
+        m_plasmaShellSurface->setSkipTaskbar(true);
+        m_plasmaShellSurface->setSkipSwitcher(true);
+    }
+
+    KX11Extras::setOnAllDesktops(winId(), Settings::showOnAllDesktops());
     KWindowEffects::enableBlurBehind(windowHandle(), m_sessionStack->wantsBlur());
 }
 
@@ -1125,9 +1131,10 @@ void MainWindow::wmActiveWindowChanged()
         return;
     }
 
-    KWindowInfo info(KWindowSystem::activeWindow(), {}, NET::WM2TransientFor);
+    const QWindow *focusWindow = QGuiApplication::focusWindow();
 
-    if (info.valid() && info.transientFor() == winId()) {
+    // Don't retract when opening one of our config windows
+    if (focusWindow && focusWindow->transientParent() == windowHandle()) {
         return;
     }
 
@@ -1184,7 +1191,7 @@ void MainWindow::_toggleWindowState()
 {
     bool visible = isVisible();
 
-    if (visible && KWindowSystem::activeWindow() != winId() && Settings::keepOpen()) {
+    if (visible && !isActiveWindow() && Settings::keepOpen()) {
         // Window is open but doesn't have focus; it's set to stay open
         // regardless of focus loss.
 
@@ -1195,11 +1202,11 @@ void MainWindow::_toggleWindowState()
             // will also cause the window manager to switch to the virtual
             // desktop the window resides on.
 
-            KWindowSystem::activateWindow(winId());
-            KWindowSystem::forceActiveWindow(winId());
+            KWindowSystem::activateWindow(windowHandle());
+            KX11Extras::forceActiveWindow(winId());
 
             return;
-        } else if (!Settings::showOnAllDesktops() && KWindowInfo(winId(), NET::WMDesktop).desktop() != KWindowSystem::currentDesktop()) {
+        } else if (!Settings::showOnAllDesktops() && KWindowInfo(winId(), NET::WMDesktop).desktop() != KX11Extras::currentDesktop()) {
             // The open/restrict action isn't set to focus the window, but
             // the window is currently on another virtual desktop (the option
             // to show it on all of them is disabled), so closing it doesn't
@@ -1209,10 +1216,10 @@ void MainWindow::_toggleWindowState()
             // switch to the virtual desktop the window currently resides on,
             // so move the window to the current desktop before doing so.
 
-            KWindowSystem::setOnDesktop(winId(), KWindowSystem::currentDesktop());
+            KX11Extras::setOnDesktop(winId(), KX11Extras::currentDesktop());
 
-            KWindowSystem::activateWindow(winId());
-            KWindowSystem::forceActiveWindow(winId());
+            KWindowSystem::activateWindow(windowHandle());
+            KX11Extras::forceActiveWindow(winId());
 
             return;
         }
@@ -1222,7 +1229,7 @@ void MainWindow::_toggleWindowState()
     if (!Settings::useWMAssist() && m_kwinAssistPropSet)
         kwinAssistPropCleanup();
 
-    if (m_isX11 && Settings::useWMAssist() && KWindowSystem::compositingActive())
+    if (m_isX11 && Settings::useWMAssist() && KX11Extras::compositingActive())
         kwinAssistToggleWindowState(visible);
     else
 #endif
@@ -1409,7 +1416,7 @@ void MainWindow::sharedPreOpenWindow()
 void MainWindow::sharedAfterOpenWindow()
 {
     if (!Settings::firstRun())
-        KWindowSystem::forceActiveWindow(winId());
+        KX11Extras::forceActiveWindow(winId());
 
     connect(qGuiApp, &QGuiApplication::focusWindowChanged, this, &MainWindow::wmActiveWindowChanged);
 
@@ -1442,7 +1449,7 @@ void MainWindow::sharedAfterHideWindow()
 
 void MainWindow::activate()
 {
-    KWindowSystem::activateWindow(winId());
+    KWindowSystem::activateWindow(windowHandle());
 }
 
 void MainWindow::toggleMousePoll(bool poll)
@@ -1541,7 +1548,7 @@ QRect MainWindow::getDesktopGeometry()
     }
 
     if (QGuiApplication::screens().count() > 1) {
-        const QList<WId> allWindows = KWindowSystem::windows();
+        const QList<WId> allWindows = KX11Extras::windows();
         QList<WId> offScreenWindows;
 
         QListIterator<WId> i(allWindows);
@@ -1549,7 +1556,7 @@ QRect MainWindow::getDesktopGeometry()
         while (i.hasNext()) {
             WId windowId = i.next();
 
-            if (KWindowSystem::hasWId(windowId)) {
+            if (KX11Extras::hasWId(windowId)) {
                 KWindowInfo windowInfo = KWindowInfo(windowId, NET::WMDesktop | NET::WMGeometry, NET::WM2ExtendedStrut);
 
                 // If windowInfo is valid and the window is located at the same (current)
@@ -1588,10 +1595,14 @@ QRect MainWindow::getDesktopGeometry()
             }
         }
 
-        return KWindowSystem::workArea(offScreenWindows).intersected(screenGeometry);
+        return KX11Extras::workArea(offScreenWindows).intersected(screenGeometry);
     }
 
-    return KWindowSystem::workArea();
+#if HAVE_X11
+    return KX11Extras::workArea();
+#else
+    return QRect();
+#endif
 }
 
 void MainWindow::whatsThis()
@@ -1618,7 +1629,7 @@ void MainWindow::firstRunDialogFinished()
 
     m_firstRunDialog->deleteLater();
 
-    KWindowSystem::forceActiveWindow(winId());
+    KX11Extras::forceActiveWindow(winId());
 }
 
 void MainWindow::firstRunDialogOk()
@@ -1632,7 +1643,7 @@ void MainWindow::firstRunDialogOk()
 
 void MainWindow::updateUseTranslucency()
 {
-    m_useTranslucency = (Settings::translucency() && KWindowSystem::compositingActive());
+    m_useTranslucency = (Settings::translucency() && (m_isX11 ? KX11Extras::compositingActive() : true));
 }
 
 void MainWindow::updateTrayTooltip()
