@@ -141,23 +141,25 @@ Terminal *Session::addTerminal(QWidget *parent, QString workingDir)
         workingDir = m_workingDir;
     }
 
-    Terminal *terminal = new Terminal(workingDir, parent);
-    connect(terminal, SIGNAL(activated(int)), this, SLOT(setActiveTerminal(int)));
-    connect(terminal, SIGNAL(manuallyActivated(Terminal *)), this, SIGNAL(terminalManuallyActivated(Terminal *)));
-    connect(terminal, SIGNAL(titleChanged(int, QString)), this, SLOT(setTitle(int, QString)));
-    connect(terminal, SIGNAL(keyboardInputBlocked(Terminal *)), this, SIGNAL(keyboardInputBlocked(Terminal *)));
-    connect(terminal, SIGNAL(silenceDetected(Terminal *)), this, SIGNAL(silenceDetected(Terminal *)));
-    connect(terminal, SIGNAL(destroyed(int)), this, SLOT(cleanup(int)));
+    std::unique_ptr<Terminal> terminal = std::make_unique<Terminal>(workingDir, parent);
+    connect(terminal.get(), SIGNAL(activated(int)), this, SLOT(setActiveTerminal(int)));
+    connect(terminal.get(), SIGNAL(manuallyActivated(Terminal *)), this, SIGNAL(terminalManuallyActivated(Terminal *)));
+    connect(terminal.get(), SIGNAL(titleChanged(int, QString)), this, SLOT(setTitle(int, QString)));
+    connect(terminal.get(), SIGNAL(keyboardInputBlocked(Terminal *)), this, SIGNAL(keyboardInputBlocked(Terminal *)));
+    connect(terminal.get(), SIGNAL(silenceDetected(Terminal *)), this, SIGNAL(silenceDetected(Terminal *)));
+    connect(terminal.get(), &Terminal::closeRequested, this, QOverload<int>::of(&Session::cleanup));
 
-    m_terminals.insert(terminal->id(), terminal);
+    Terminal *term = terminal.get();
+
+    m_terminals[terminal->id()] = std::move(terminal);
 
     Q_EMIT wantsBlurChanged();
 
-    QWidget *terminalWidget = terminal->terminalWidget();
+    QWidget *terminalWidget = term->terminalWidget();
     if (terminalWidget)
         terminalWidget->setFocus();
 
-    return terminal;
+    return term;
 }
 
 void Session::closeTerminal(int terminalId)
@@ -169,7 +171,7 @@ void Session::closeTerminal(int terminalId)
     if (!m_terminals.contains(terminalId))
         return;
 
-    m_terminals.value(terminalId)->deletePart();
+    cleanup(terminalId);
 }
 
 void Session::focusPreviousTerminal()
@@ -179,28 +181,19 @@ void Session::focusPreviousTerminal()
     if (!m_terminals.contains(m_activeTerminalId))
         return;
 
-    QMapIterator<int, Terminal *> it(m_terminals);
+    std::map<int, std::unique_ptr<Terminal>>::iterator currentTerminal = m_terminals.find(m_activeTerminalId);
 
-    it.toBack();
+    std::map<int, std::unique_ptr<Terminal>>::iterator previousTerminal;
 
-    while (it.hasPrevious()) {
-        it.previous();
+    if (currentTerminal == m_terminals.begin()) {
+        previousTerminal = std::prev(m_terminals.end());
+    } else {
+        previousTerminal = std::prev(currentTerminal);
+    }
 
-        if (it.key() == m_activeTerminalId) {
-            if (it.hasPrevious()) {
-                QWidget *terminalWidget = it.peekPrevious().value()->terminalWidget();
-                if (terminalWidget)
-                    terminalWidget->setFocus();
-            } else {
-                it.toBack();
-
-                QWidget *terminalWidget = it.peekPrevious().value()->terminalWidget();
-                if (terminalWidget)
-                    terminalWidget->setFocus();
-            }
-
-            break;
-        }
+    QWidget *terminalWidget = previousTerminal->second->terminalWidget();
+    if (terminalWidget) {
+        terminalWidget->setFocus();
     }
 }
 
@@ -211,26 +204,17 @@ void Session::focusNextTerminal()
     if (!m_terminals.contains(m_activeTerminalId))
         return;
 
-    QMapIterator<int, Terminal *> it(m_terminals);
+    std::map<int, std::unique_ptr<Terminal>>::iterator currentTerminal = m_terminals.find(m_activeTerminalId);
 
-    while (it.hasNext()) {
-        it.next();
+    std::map<int, std::unique_ptr<Terminal>>::iterator nextTerminal = std::next(currentTerminal);
 
-        if (it.key() == m_activeTerminalId) {
-            if (it.hasNext()) {
-                QWidget *terminalWidget = it.peekNext().value()->terminalWidget();
-                if (terminalWidget)
-                    terminalWidget->setFocus();
-            } else {
-                it.toFront();
+    if (nextTerminal == m_terminals.end()) {
+        nextTerminal = m_terminals.begin();
+    }
 
-                QWidget *terminalWidget = it.peekNext().value()->terminalWidget();
-                if (terminalWidget)
-                    terminalWidget->setFocus();
-            }
-
-            break;
-        }
+    QWidget *terminalWidget = nextTerminal->second->terminalWidget();
+    if (terminalWidget) {
+        terminalWidget->setFocus();
     }
 }
 
@@ -243,7 +227,7 @@ int Session::splitLeftRight(int terminalId)
     if (!m_terminals.contains(terminalId))
         return -1;
 
-    Terminal *terminal = m_terminals.value(terminalId);
+    Terminal *terminal = m_terminals[terminalId].get();
 
     if (terminal)
         return split(terminal, Qt::Horizontal);
@@ -260,7 +244,7 @@ int Session::splitTopBottom(int terminalId)
     if (!m_terminals.contains(terminalId))
         return -1;
 
-    Terminal *terminal = m_terminals.value(terminalId);
+    Terminal *terminal = m_terminals[terminalId].get();
 
     if (terminal)
         return split(terminal, Qt::Vertical);
@@ -362,7 +346,7 @@ void Session::setActiveTerminal(int terminalId)
 {
     m_activeTerminalId = terminalId;
 
-    setTitle(m_activeTerminalId, m_terminals.value(m_activeTerminalId)->title());
+    setTitle(m_activeTerminalId, m_terminals[m_activeTerminalId]->title());
 }
 
 void Session::setTitle(int terminalId, const QString &title)
@@ -377,10 +361,11 @@ void Session::setTitle(int terminalId, const QString &title)
 
 void Session::cleanup(int terminalId)
 {
-    if (m_activeTerminalId == terminalId && m_terminals.count() > 1)
+    if (m_activeTerminalId == terminalId && m_terminals.size() > 1)
         focusPreviousTerminal();
 
-    m_terminals.remove(terminalId);
+    m_terminals.erase(terminalId);
+    Q_EMIT wantsBlurChanged();
 
     cleanup();
 }
@@ -392,7 +377,7 @@ void Session::cleanup()
 
     m_baseSplitter->recursiveCleanup();
 
-    if (m_terminals.isEmpty())
+    if (m_terminals.empty())
         m_baseSplitter->deleteLater();
 }
 
@@ -405,13 +390,10 @@ void Session::prepareShutdown()
 
 const QString Session::terminalIdList()
 {
-    QList<int> keyList = m_terminals.keys();
     QStringList idList;
-
-    QListIterator<int> i(keyList);
-
-    while (i.hasNext())
-        idList << QString::number(i.next());
+    for (auto &[id, terminal] : m_terminals) {
+        idList << QString::number(id);
+    }
 
     return idList.join(QLatin1Char(','));
 }
@@ -426,7 +408,7 @@ Terminal *Session::getTerminal(int terminalId)
     if (!m_terminals.contains(terminalId))
         return nullptr;
 
-    return m_terminals.value(terminalId);
+    return m_terminals[terminalId].get();
 }
 
 void Session::runCommand(const QString &command, int terminalId)
@@ -438,7 +420,7 @@ void Session::runCommand(const QString &command, int terminalId)
     if (!m_terminals.contains(terminalId))
         return;
 
-    m_terminals.value(terminalId)->runCommand(command);
+    m_terminals[terminalId]->runCommand(command);
 }
 
 void Session::manageProfiles()
@@ -448,7 +430,7 @@ void Session::manageProfiles()
     if (!m_terminals.contains(m_activeTerminalId))
         return;
 
-    m_terminals.value(m_activeTerminalId)->manageProfiles();
+    m_terminals[m_activeTerminalId]->manageProfiles();
 }
 
 void Session::editProfile()
@@ -458,28 +440,22 @@ void Session::editProfile()
     if (!m_terminals.contains(m_activeTerminalId))
         return;
 
-    m_terminals.value(m_activeTerminalId)->editProfile();
+    m_terminals[m_activeTerminalId]->editProfile();
 }
 
 bool Session::keyboardInputEnabled()
 {
-    int keyboardInputEnabledCount = 0;
-
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        if (i.next().value()->keyboardInputEnabled())
-            ++keyboardInputEnabledCount;
-
-    return m_terminals.count() == keyboardInputEnabledCount;
+    return std::all_of(m_terminals.cbegin(), m_terminals.cend(), [](auto &it) {
+        auto &[id, terminal] = it;
+        return terminal->keyboardInputEnabled();
+    });
 }
 
 void Session::setKeyboardInputEnabled(bool enabled)
 {
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        setKeyboardInputEnabled(i.next().key(), enabled);
+    for (auto &[id, terminal] : m_terminals) {
+        terminal->setKeyboardInputEnabled(enabled);
+    }
 }
 
 bool Session::keyboardInputEnabled(int terminalId)
@@ -487,7 +463,7 @@ bool Session::keyboardInputEnabled(int terminalId)
     if (!m_terminals.contains(terminalId))
         return false;
 
-    return m_terminals.value(terminalId)->keyboardInputEnabled();
+    return m_terminals[terminalId]->keyboardInputEnabled();
 }
 
 void Session::setKeyboardInputEnabled(int terminalId, bool enabled)
@@ -495,50 +471,38 @@ void Session::setKeyboardInputEnabled(int terminalId, bool enabled)
     if (!m_terminals.contains(terminalId))
         return;
 
-    m_terminals.value(terminalId)->setKeyboardInputEnabled(enabled);
+    m_terminals[terminalId]->setKeyboardInputEnabled(enabled);
 }
 
 bool Session::hasTerminalsWithKeyboardInputEnabled()
 {
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        if (i.next().value()->keyboardInputEnabled())
-            return true;
-
-    return false;
+    return std::any_of(m_terminals.cbegin(), m_terminals.cend(), [](auto &it) {
+        auto &[id, terminal] = it;
+        return terminal->keyboardInputEnabled();
+    });
 }
 
 bool Session::hasTerminalsWithKeyboardInputDisabled()
 {
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        if (!i.next().value()->keyboardInputEnabled())
-            return true;
-
-    return false;
+    return std::any_of(m_terminals.cbegin(), m_terminals.cend(), [](auto &it) {
+        auto &[id, terminal] = it;
+        return !terminal->keyboardInputEnabled();
+    });
 }
 
 bool Session::monitorActivityEnabled()
 {
-    int monitorActivityEnabledCount = 0;
-
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        if (i.next().value()->monitorActivityEnabled())
-            ++monitorActivityEnabledCount;
-
-    return m_terminals.count() == monitorActivityEnabledCount;
+    return std::all_of(m_terminals.cbegin(), m_terminals.cend(), [](auto &it) {
+        auto &[id, terminal] = it;
+        return terminal->monitorActivityEnabled();
+    });
 }
 
 void Session::setMonitorActivityEnabled(bool enabled)
 {
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        setMonitorActivityEnabled(i.next().key(), enabled);
+    for (auto &[id, terminal] : m_terminals) {
+        setMonitorActivityEnabled(id, enabled);
+    }
 }
 
 bool Session::monitorActivityEnabled(int terminalId)
@@ -546,7 +510,7 @@ bool Session::monitorActivityEnabled(int terminalId)
     if (!m_terminals.contains(terminalId))
         return false;
 
-    return m_terminals.value(terminalId)->monitorActivityEnabled();
+    return m_terminals[terminalId]->monitorActivityEnabled();
 }
 
 void Session::setMonitorActivityEnabled(int terminalId, bool enabled)
@@ -554,7 +518,7 @@ void Session::setMonitorActivityEnabled(int terminalId, bool enabled)
     if (!m_terminals.contains(terminalId))
         return;
 
-    Terminal *terminal = m_terminals.value(terminalId);
+    Terminal *terminal = m_terminals[terminalId].get();
 
     connect(terminal, SIGNAL(activityDetected(Terminal *)), this, SIGNAL(activityDetected(Terminal *)), Qt::UniqueConnection);
 
@@ -563,57 +527,42 @@ void Session::setMonitorActivityEnabled(int terminalId, bool enabled)
 
 bool Session::hasTerminalsWithMonitorActivityEnabled()
 {
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        if (i.next().value()->monitorActivityEnabled())
-            return true;
-
-    return false;
+    return std::any_of(m_terminals.cbegin(), m_terminals.cend(), [](auto &it) {
+        auto &[id, terminal] = it;
+        return terminal->monitorActivityEnabled();
+    });
 }
 
 bool Session::hasTerminalsWithMonitorActivityDisabled()
 {
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        if (!i.next().value()->monitorActivityEnabled())
-            return true;
-
-    return false;
+    return std::any_of(m_terminals.cbegin(), m_terminals.cend(), [](auto &it) {
+        auto &[id, terminal] = it;
+        return !terminal->monitorActivityEnabled();
+    });
 }
 
 void Session::reconnectMonitorActivitySignals()
 {
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext()) {
-        i.next();
+    for (auto &[id, terminal] : m_terminals) {
         // clang-format off
-        connect(i.value(), SIGNAL(activityDetected(Terminal*)), this, SIGNAL(activityDetected(Terminal*)), Qt::UniqueConnection);
+        connect(terminal.get(), SIGNAL(activityDetected(Terminal*)), this, SIGNAL(activityDetected(Terminal*)), Qt::UniqueConnection);
         // clang-format on
     }
 }
 
 bool Session::monitorSilenceEnabled()
 {
-    int monitorSilenceEnabledCount = 0;
-
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        if (i.next().value()->monitorSilenceEnabled())
-            ++monitorSilenceEnabledCount;
-
-    return m_terminals.count() == monitorSilenceEnabledCount;
+    return std::all_of(m_terminals.cbegin(), m_terminals.cend(), [](auto &it) {
+        auto &[id, terminal] = it;
+        return terminal->monitorSilenceEnabled();
+    });
 }
 
 void Session::setMonitorSilenceEnabled(bool enabled)
 {
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        setMonitorSilenceEnabled(i.next().key(), enabled);
+    for (auto &[id, terminal] : m_terminals) {
+        terminal->setMonitorSilenceEnabled(enabled);
+    }
 }
 
 bool Session::monitorSilenceEnabled(int terminalId)
@@ -621,7 +570,7 @@ bool Session::monitorSilenceEnabled(int terminalId)
     if (!m_terminals.contains(terminalId))
         return false;
 
-    return m_terminals.value(terminalId)->monitorSilenceEnabled();
+    return m_terminals[terminalId]->monitorSilenceEnabled();
 }
 
 void Session::setMonitorSilenceEnabled(int terminalId, bool enabled)
@@ -629,34 +578,29 @@ void Session::setMonitorSilenceEnabled(int terminalId, bool enabled)
     if (!m_terminals.contains(terminalId))
         return;
 
-    m_terminals.value(terminalId)->setMonitorSilenceEnabled(enabled);
+    m_terminals[terminalId]->setMonitorSilenceEnabled(enabled);
 }
 
 bool Session::hasTerminalsWithMonitorSilenceDisabled()
 {
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        if (!i.next().value()->monitorSilenceEnabled())
-            return true;
-
-    return false;
+    return std::any_of(m_terminals.cbegin(), m_terminals.cend(), [](auto &it) {
+        auto &[id, terminal] = it;
+        return !terminal->monitorSilenceEnabled();
+    });
 }
 
 bool Session::hasTerminalsWithMonitorSilenceEnabled()
 {
-    QMapIterator<int, Terminal *> i(m_terminals);
-
-    while (i.hasNext())
-        if (i.next().value()->monitorSilenceEnabled())
-            return true;
-
-    return false;
+    return std::any_of(m_terminals.cbegin(), m_terminals.cend(), [](auto &it) {
+        auto &[id, terminal] = it;
+        return terminal->monitorSilenceEnabled();
+    });
 }
 
 bool Session::wantsBlur() const
 {
-    return std::any_of(m_terminals.cbegin(), m_terminals.cend(), [](Terminal *term) {
-        return term->wantsBlur();
+    return std::any_of(m_terminals.cbegin(), m_terminals.cend(), [](auto &it) {
+        auto &[id, terminal] = it;
+        return terminal->wantsBlur();
     });
 }
